// $Id: curlypage_admin.js,v 1.3 2010/06/10 11:15:44 manfer Exp $
(function ($) {

/**
 * Provide the summary information for the curlypage settings vertical tabs.
 */
Drupal.behaviors.curlypageSettingsSummary = {
  attach: function (context) {
    // The drupalSetSummary method required for this behavior is not available
    // on the Curlypages administration page, so we need to make sure this
    // behavior is processed only if drupalSetSummary is defined.
    if (typeof jQuery.fn.drupalSetSummary == 'undefined') {
      return;
    }

    $('fieldset#edit-position-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      var corner = $('input[name="peel_position"]:checked').next('label').text();
      vals.push(corner);
      var model = $('input[name="peel_position_model"]:checked').next('label').text();
      vals.push(model);
      return vals.join(', ');
    });

    $('fieldset#edit-wait-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      if (!$('input[name="wait_enable"]:checked', context).val()) {
        return Drupal.t('Disabled');
      }
      return Drupal.t('Enabled');
    });

    $('fieldset#edit-style-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      var flagStyle = $('select[name="flag_style"] option:selected').text();
      vals.push(Drupal.t('Flag: @value', { '@value': flagStyle }));
      var peelStyle = $('select[name="peel_style"] option:selected').text();
      vals.push(Drupal.t('Peel: @value', { '@value': peelStyle }));
      return vals.join('<br />');
    });

    $('fieldset#edit-size-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      var flagDimension = $('input[name="flag_width"]').val() + 'x' + $('input[name="flag_height"]').val();
      vals.push(Drupal.t('Flag: @value', { '@value' : flagDimension }));
      var peelDimension = $('input[name="peel_width"]').val() + 'x' + $('input[name="peel_height"]').val();
      vals.push(Drupal.t('Peel: @value', { '@value' : peelDimension }));
      return vals.join('<br />');
    });

    $('fieldset#edit-image-settings', context).drupalSetSummary(function (context) {
      var vals = [];

      if ($('input[name="mirror"]:checked').val()) {
        vals.push(Drupal.t('Back mirror enabled'));
      }

      var transition = $('select[name="in_transition"] option:selected').text();
      var duration = $('input[name="transition_duration"]').val();
      vals.push(Drupal.t('Transition: @value1 / @value2', { '@value1' : transition , '@value2' : duration }));

      var style = $('input[name="peel_color_style"]:checked').next('label').text();
      var color = $('input[name="peel_color"]:checked').next('label').text();
      var isCustom = $('input[name="peel_color"]:checked').val() == 'custom';
      var custom = $('input[name="custom_color"]').val();

      if (isCustom) {
        vals.push(Drupal.t('Color: @value1 @value2 @value3', { '@value1' : style , '@value2' : color , '@value3' : custom }));
      } else {
        vals.push(Drupal.t('Color: @value1 @value2', { '@value1' : style , '@value2' : color }));
      }

      return vals.join("<br />");
    });

    $('fieldset#edit-sound-settings', context).drupalSetSummary(function (context) {
      var vals = [];

      if ($('input[name="load_sound_remove"]').length) {
        vals.push(Drupal.t('Load sound enabled'));
      }
      if ($('input[name="open_sound_remove"]').length) {
        vals.push(Drupal.t('Open sound enabled'));
      }
      if ($('input[name="close_sound_remove"]').length) {
        vals.push(Drupal.t('Close sound enabled'));
      }
      if (!vals.length) {
        return Drupal.t('No sounds');
      }
      return vals.join('<br />');
    });

    $('fieldset#edit-linking-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      if (!$('input[name="link_enabled"]:checked', context).val()) {
        return Drupal.t('Disabled');
      }
      var link = $('input[name="link"]').val();
      vals.push(Drupal.t('URL: @value', { '@value': link }));
      var target = $('input[name="link_target"]:checked').next('label').text();
      vals.push(Drupal.t('Target: @value', { '@value': target }));
      return vals.join('<br />');
    });

    $('fieldset#edit-motion-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      var flagSpeed = $('input[name="flag_speed"]').val();
      vals.push(Drupal.t('Flag Speed: @value', { '@value': flagSpeed }));
      var peelSpeed = $('input[name="peel_speed"]').val();
      vals.push(Drupal.t('Peel Speed: @value', { '@value': peelSpeed }));
      var automaticOpen = $('input[name="automatic_open"]').val();
      vals.push(Drupal.t('Automatic Open: @value sec.', { '@value': automaticOpen }));
      var automaticClose = $('input[name="automatic_close"]').val();
      vals.push(Drupal.t('Automatic Close: @value sec.', { '@value': automaticClose }));
      return vals.join('<br />');
    });

    $('fieldset#edit-closebutton-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      if (!$('input[type="checkbox"]:checked', context).val()) {
        return Drupal.t('Disabled');
      }
      var text = $('input[name="text_on_close_button"]').val();
      vals.push(Drupal.t('Text: @value', { '@value': text }));
      var color = $('input[name="close_button_color"]').val();
      vals.push(Drupal.t('Color: @value', { '@value': color }));
      return vals.join('<br />');
    });

    $('fieldset#edit-time-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      var delay = $('input[name="delay"]').val();
      vals.push(Drupal.t('Delay: @value sec.', { '@value': delay }));
      var timeSlot = $('input[name="time_slot"]').val();
      vals.push(Drupal.t('Time Slot: @value sec.', { '@value': timeSlot }));
      var repeatTimes = $('input[name="repeat_times"]').val();
      vals.push(Drupal.t('Repeat: @value times', { '@value': repeatTimes }));
      return vals.join('<br />');
    });

    $('fieldset#edit-page-vis-settings', context).drupalSetSummary(function (context) {
      if (!$('textarea[name="pages"]', context).val()) {
        return Drupal.t('Not restricted');
      }
      else {
        return Drupal.t('Restricted to certain pages');
      }
    });

    $('fieldset#edit-node-types-vis-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      $('input[type="checkbox"]:checked', context).each(function () {
        vals.push($.trim($(this).next('label').text()));
      });
      if (!vals.length) {
        return Drupal.t('Not restricted');
      }
      return Drupal.t('Show on:') + " " + vals.join(', ');
    });

    $('fieldset#edit-languages-vis-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      $('input[type="checkbox"]:checked', context).each(function () {
        vals.push($.trim($(this).next('label').text()));
      });
      if (!vals.length) {
        return Drupal.t('Not restricted');
      }
      return vals.join(', ');
    });

    $('fieldset#edit-roles-vis-settings', context).drupalSetSummary(function (context) {
      var vals = [];
      $('input[type="checkbox"]:checked', context).each(function () {
        vals.push($.trim($(this).next('label').text()));
      });
      if (!vals.length) {
        return Drupal.t('Not restricted');
      }
      var $radio = $('input[name="roles_visibility"]:checked', context);
      if ($radio.val() == 0) {
        return Drupal.t('Show to:') + " " + vals.join(', ');
      }
      else {
        return Drupal.t('Show except to:') + " " + vals.join(', ');
      }
    });

  }
};

})(jQuery);
